Attribute VB_Name = "mListview"
Option Explicit
'
' Copyright  1997-1999 Brad Martinez, http://www.mvps.org
'
' =========================================================
' user-defined listview definitions

' Value returned by many listview messages indicating error
' (the index of no listview item)
Public Const LVI_NOITEM = -1

' Listview item state image index values
Public Enum LVItemStates
  lvisNoButton = 0
  lvisCollapsed = 1
  lvisExpanded = 2
End Enum

' Listview item overlay icon index values
Public Enum LVItemOverlays
  lvioNoOverlay = 0
  lvioShare = 1
  lvioShortcut = 2
End Enum

' ListView_GetRelativeItem flags
Public Enum LVRelativeItemFlags
  lvriParent = 0
  lvriChild = 1
  lvriFirstSibling = 2
  lvriLastSibling = 3
  lvriPrevSibling = 4
  lvriNextSibling = 5
End Enum

' ListView_GetItemCountEx flags
Public Enum LVItemCountFlags
  lvicParents = 0
  lvrcChildren = 1
  lvicSiblings = 2
End Enum

' =========================================================
' listview definitions

' we're using IE3 definitions...
#Const WIN32_IE = &H300

' styles
Public Const LVS_SHAREIMAGELISTS = &H40
Public Const LVS_NOLABELWRAP = &H80
Public Const LVS_NOCOLUMNHEADER = &H4000

' messages
Public Const LVM_FIRST = &H1000
Public Const LVM_SETIMAGELIST = (LVM_FIRST + 3)
Public Const LVM_GETITEMCOUNT = (LVM_FIRST + 4)
Public Const LVM_GETITEM = (LVM_FIRST + 5)
Public Const LVM_SETITEM = (LVM_FIRST + 6)
Public Const LVM_GETNEXTITEM = (LVM_FIRST + 12)
Public Const LVM_GETITEMRECT = (LVM_FIRST + 14)
Public Const LVM_GETSTRINGWIDTH = (LVM_FIRST + 17)
Public Const LVM_HITTEST = (LVM_FIRST + 18)
Public Const LVM_ENSUREVISIBLE = (LVM_FIRST + 19)
Public Const LVM_SETCOLUMNWIDTH = (LVM_FIRST + 30)
Public Const LVM_SETITEMSTATE = (LVM_FIRST + 43)
Public Const LVM_SORTITEMS = (LVM_FIRST + 48)

' LVM_SETIMAGELIST wParam
Public Const LVSIL_SMALL = 1
Public Const LVSIL_STATE = 2

' LVM_GET/SETITEM lParam
Public Type LVITEM   ' was LV_ITEM
  mask As Long
  iItem As Long
  iSubItem As Long
  state As Long
  stateMask As Long
  pszText As Long  ' if String, must be pre-allocated before before filled
  cchTextMax As Long
  iImage As Long
  lParam As Long
#If (WIN32_IE >= &H300) Then
  iIndent As Long
#End If
End Type

' LVITEM mask
Public Const LVIF_TEXT = &H1
Public Const LVIF_IMAGE = &H2
Public Const LVIF_STATE = &H8
#If (WIN32_IE >= &H300) Then
Public Const LVIF_INDENT = &H10
#End If
 
' LVITEM state, stateMask
Public Const LVIS_FOCUSED = &H1
Public Const LVIS_SELECTED = &H2
Public Const LVIS_OVERLAYMASK = &HF00
Public Const LVIS_STATEIMAGEMASK = &HF000

' LVM_GETNEXTITEM lParam
Public Const LVNI_ALL = &H0
Public Const LVNI_FOCUSED = &H1
Public Const LVNI_SELECTED = &H2

' LVM_GETITEMRECT rct.Left (lParam)
Public Const LVIR_LABEL = 2

' LVM_HITTEST lParam
Public Type LVHITTESTINFO   ' was LV_HITTESTINFO
  pt As POINTAPI
  flags As Long
  iItem As Long
#If (WIN32_IE >= &H300) Then
  iSubItem As Long    ' this is was NOT in win95.  valid only for LVM_SUBITEMHITTEST
#End If
End Type
 
' LVHITTESTINFO flags
Public Const LVHT_ONITEMICON = &H2
Public Const LVHT_ONITEMLABEL = &H4
Public Const LVHT_ONITEMSTATEICON = &H8

' LVM_SETCOLUMNWIDTH lParam
'Public Const LVSCW_AUTOSIZE = -1

' notifications
Public Const LVN_FIRST = -100&  ' (0U-100U)
Public Const LVN_ITEMCHANGING = (LVN_FIRST - 0)   ' lParam = NMLISTVIEW, rtn T/F
Public Const LVN_ITEMCHANGED = (LVN_FIRST - 1)    ' lParam = NMLISTVIEW, no rtn

' lParam for most listview notification messages
Public Type NMLISTVIEW   ' was NM_LISTVIEW
  hdr As NMHDR
  iItem As Long
  iSubItem As Long
  uNewState As Long
  uOldState As Long
  uChanged As Long
  ptAction As POINTAPI
  lParam As Long
End Type
'

' ==============================================================
' listview calls

' Inserts a new root folder into the listview control.
'
'   objLV  - ListView object reference
'   sPath  - root item's fully qualified path
'
' If successful, returns the root folder's VB ListView ListItem Index, returns 0 otherwise.
'
' Called only from Form1.Drive1_Change

Public Sub InsertRootFolder(objLV As ListView, sPath As String)
  
  Call objLV.ListItems.Clear
  
  ' Insert the root and select it.
  Call InsertFolder(objLV, 0, 0, sPath)
  objLV.ListItems(1).Selected = True
  
  ' Expand the root and add the root's subfolders.
  Call InsertSubfolders(objLV, 0, 0, sPath)

End Sub

' Inserts the specified folder at the specified Index (position) in the ListView.
'
'   objLV    - ListView object reference
'   iItem      - folder's real listview item index (is a value of 1 less than its ListItem.Index)
'   iIndent   - folder's zero-based indent
'   sPath    - folder's fully qualified path

' Returns True successful, or false otherwise.

' Called from InsertRootFolder above, and InsertSubfolders below.

Public Function InsertFolder(objLV As ListView, _
                                            iItem As Long, _
                                            iIndent As Long, _
                                            sPath As String) As Boolean
  Dim ulAttrs As Long
  Dim lvi As LVITEM
  Dim li As ListItem
  
  ' Get the folder's attributes
  ulAttrs = GetFileAttributes(sPath)
  
  ' Though FindFirstFile will enumerate virtual folders (i.e. History subfolders)
  ' SHGetFileInfo can only evaluate virtual folders by their PIDLs. Since we're
  ' not doing PIDLs, we'll bail in this situation....
  If (ulAttrs = 0) Then Exit Function
  
  ' Indicate what LVITEM members will contain data
  lvi.mask = LVIF_IMAGE Or LVIF_STATE Or LVIF_INDENT
  
  ' Set the new item's real listview index and indent
  lvi.iItem = iItem
  lvi.iIndent = iIndent
    
  ' Explicitly set the folder's normal icon index, overriding
  ' the I_IMAGECALLBACK value the VB ListView uses.
  ' The selected icon index is set on LVN_ITEMCHANGED
  lvi.iImage = GetFileIconIndex(sPath, SHGFI_SMALLICON)
  
  ' If the folder has subfolders, explicitly give the item a button
  If (ulAttrs And SFGAO_HASSUBFOLDER) Then
    lvi.state = LVIS_STATEIMAGEMASK
    lvi.stateMask = INDEXTOSTATEIMAGEMASK(lvisCollapsed)
  End If

  ' If the folder is shared, give the item a share overlay icon.
  If (ulAttrs And SFGAO_SHARE) Then
    lvi.mask = lvi.mask Or LVIF_STATE
    lvi.state = lvi.state Or LVIS_OVERLAYMASK
    ' share overlay icon is the 1st system imagelist overlay image,
    ' shortcut is 2nd, 3rd and 4th images vary.
    lvi.stateMask = lvi.stateMask Or INDEXTOOVERLAYMASK(lvioShare)
  End If
  
  ' Add the new ListItem
  Set li = objLV.ListItems.Add(iItem + 1, , GetFileDisplayName(sPath))
  
  ' And set the item's button, icons (and text), done deal...
  InsertFolder = ListView_SetItem(objLV.hWnd, lvi)

  ' Since we're using drive's displaynames instead of their paths for
  ' ListItem.Text, we'll store the item's path in the ListItem's Tag
  li.Tag = sPath
  
End Function

' Inserts subfolders under the specified parent folder.

'   objLV              - ListView object reference
'   iParentItem     - parent folder's real listview item index (is a value of 1 less than its ListItem.Index)
'   iParentIndent  - parent folder's zero-based indent
'   sParentPath    - parent folder's fully qualified path

' Returns the number of subfolders inserted under the parent folder.

' Called from InsertRootFolder, Form1.ListView1_MouseDown,
' Form1.ListView1_DblClick, Form1.ListView1_KeyDown

Public Function InsertSubfolders(objLV As ListView, _
                                                    iParentItem As Long, _
                                                    iParentIndent As Long, _
                                                    ByVal sParentPath As String) As Long
  Dim wfd As WIN32_FIND_DATA
  Dim hFind As Long
  Dim asChildren() As String
  Dim nChildren As Long
  Dim i As Long
  
  Screen.MousePointer = vbHourglass
  
  ' Set the expanding parent folder's button state first.
  Call ListView_SetItemState(objLV.hWnd, iParentItem, _
                                            INDEXTOSTATEIMAGEMASK(lvisExpanded), LVIS_STATEIMAGEMASK)
  
  ' make sure the parent path has a trailing backslash...
  sParentPath = NormalizePath(sParentPath)
  
  ' Enumerate the parent folder's subfolders.
  hFind = FindFirstFile(sParentPath & vbAllFileSpec, wfd)
  If (hFind <> INVALID_HANDLE_VALUE) Then
  
    Do
      
      If (wfd.dwFileAttributes And vbDirectory) Then
        ' If not a  "." or ".." DOS subdir...
        If (Asc(wfd.cFileName) <> vbAscDot) Then
          ' Cache all subfolders under the expanding parent folders, we'll sort
          ' them before inserting them below (append the subfolder's filename
          ' to the parent folder's path).
          ReDim Preserve asChildren(nChildren)
          asChildren(nChildren) = sParentPath & Left$(wfd.cFileName, InStr(wfd.cFileName, vbNullChar) - 1)
          nChildren = nChildren + 1
        End If
      End If   ' (wfd.dwFileAttributes And vbDirectory)
      
    Loop While FindNextFile(hFind, wfd)
  
    Call FindClose(hFind)
  End If  ' (hFind <> INVALID_HANDLE_VALUE)
  
  
  If (nChildren = 0) Then
    ' If for some reason we didn't load any subfolders under the parent
    ' folder, remove the parent folder's button (could be a net folder
    ' whose SFGAO_HASSUBFOLDER was not resolvable).
    Call ListView_SetItemState(objLV.hWnd, iParentItem, lvisNoButton, LVIS_STATEIMAGEMASK)
  
  Else
    ' Sort the cached subfolders before inserting them (tried, but couldn't
    ' manage to sort only a given set of items using the listview's sort callback
    ' without completely screwing up the whole tree). This will also ensure
    ' that the Index of all ListItems correspond to their relative one based
    ' position within the ListView.
    Call ShellSortStrings(asChildren, 0, nChildren - 1, soAscending)
    ' Insert the subfolders in reverse order, giving the expanding parent the
    ' appearence of being unrolled.
    For i = nChildren - 1 To 0 Step -1
      Call InsertFolder(objLV, iParentItem + 1, iParentIndent + 1, asChildren(i))
    Next
  End If
  
  ' Autosize the listview's column width first before making folders visible
  ' below, in case the autosize causes the horizontal scrollbar to appear...
  Call ListView_AutosizeFirstColumn(objLV.hWnd)
  
  ' Make the last inserted subfolder visible, then the parent folder visible,
  ' per default treeview behavior.
  Call ListView_EnsureVisible(objLV.hWnd, iParentItem + nChildren, False)
  Call ListView_EnsureVisible(objLV.hWnd, iParentItem, False)
  
  Screen.MousePointer = vbDefault
  
  ' Return the number of inserted subfolders
  InsertSubfolders = nChildren
  
End Function

' Removes all subfolders under the specified parent folder (all ListItems
' whose indent value is greater than the specified parent Listitem).

'   objLV              - ListView object reference
'   iParentItem     - parent folder's real listview item index (is a value of 1 less than its ListItem.Index)
'   iParentIndent  - parent folder's zero-based indent

' Returns the number of all descendent subfolders removed from under
' the parent folder.

' called from Form1.ListView1_MouseDown, Form1.ListView1_DblClick, Form1.ListView1_KeyDown

Public Function RemoveSubfolders(objLV As ListView, _
                                                        iParentItem As Long, _
                                                        iParentIndent As Long) As Long
  Dim iSelItem As Long
  Dim nItemsStart As Long
  Dim nItems As Long
  Dim iIndentChild As Long
  
  Screen.MousePointer = vbHourglass
  
  ' First set the collapsing parent's button.
  Call ListView_SetItemState(objLV.hWnd, iParentItem, _
                                            INDEXTOSTATEIMAGEMASK(lvisCollapsed), LVIS_STATEIMAGEMASK)
  
  ' Get the index of the selected item, and a count of items before we start
  iSelItem = objLV.SelectedItem.Index
  nItemsStart = objLV.ListItems.Count
  nItems = nItemsStart
  
  Do
    ' If the item immediately below the collapsing parent has an indent
    ' value greater than the parent (real listview items are zero-based)...
    Call ListView_GetItemStateEx(objLV.hWnd, iParentItem + 1, iIndentChild)
    If (iIndentChild > iParentIndent) Then
      
      ' If we're about to remove the selected item, select the collapsing parent.
      If (iSelItem - (nItemsStart - nItems)) = (iParentItem + 2) Then
        Call ListView_SetSelectedItem(objLV.hWnd, iParentItem)
      End If
      
      ' Remove the item immediately below the collapsing parent, giving the
      ' parent the appearence of being rolled closed (VB ListItems are one-based)
      Call objLV.ListItems.Remove(iParentItem + 2)
      
      ' Keep a count of ListView items so we don't try to remove more
      ' items than are in the ListView (when collapsing the last parent).
      nItems = nItems - 1
    End If
  Loop While (iIndentChild > iParentIndent) And (iParentItem + 1 < nItems)
  
  ' Autosize the listview's column width.
  Call ListView_AutosizeFirstColumn(objLV.hWnd)

  Screen.MousePointer = vbDefault

  ' Return the number of removed subfolders
  RemoveSubfolders = nItemsStart - nItems
  
End Function

' =========================================================
' user-defined listview macros

' Autosizes the first listview column (LVSCW_AUTOSIZE does
' not take item indentation [LVITEM.iIndent] into consideration)

' If successful, returns the autosized width, returns 0 otherwsie.

Public Function ListView_AutosizeFirstColumn(hwndLV As Long) As Long
  Dim lvi As LVITEM
  Dim iItem As Long
  Dim rc As RECT
  Dim xMaxLabelRight As Long
  
  ' Initialize the common struct members.
  lvi.mask = LVIF_TEXT
  lvi.pszText = StrPtr(String$(MAX_PATH, 0))
  lvi.cchTextMax = MAX_PATH
  
  iItem = LVI_NOITEM
  Do
    iItem = ListView_GetNextItem(hwndLV, iItem, LVNI_ALL)
    If (iItem = LVI_NOITEM) Then Exit Do
      
    ' Get the current item's label rect
    If ListView_GetItemRect(hwndLV, iItem, rc, LVIR_LABEL) Then
      
      ' Get the current item's text in a long pointer.
      lvi.iItem = iItem
      If ListView_GetItem(hwndLV, lvi) Then
        
        ' Cache the maximum sum of the item's label rect left coord, the item's
        ' text width, and 2 pixels for the margin on either side of the label.
        xMaxLabelRight = Max(xMaxLabelRight, rc.Left + ListView_GetStringWidth(hwndLV, lvi.pszText) + 4)
      
      End If   ' ListView_GetItem
    End If   ' ListView_GetItemRect
  
  Loop
  
  If xMaxLabelRight Then
    If ListView_SetColumnWidth(hwndLV, 0, xMaxLabelRight) Then
      ListView_AutosizeFirstColumn = xMaxLabelRight
    End If
  End If

End Function

' Returns the state and indent vaues of the specified listview item

Public Function ListView_GetItemStateEx(hwndLV As Long, iItem As Long, iIndent As Long) As LVItemStates
  Dim lvi As LVITEM
  
  lvi.mask = LVIF_STATE Or LVIF_INDENT
  lvi.iItem = iItem
  lvi.stateMask = LVIS_STATEIMAGEMASK
  
  If ListView_GetItem(hwndLV, lvi) Then
    iIndent = lvi.iIndent
    ListView_GetItemStateEx = STATEIMAGEMASKTOINDEX(lvi.state And LVIS_STATEIMAGEMASK)
  End If
  
End Function

' Returns the zero-based position (index) of the item within the listview,
' that is selected and has the focus rectangle (as opposed to the item's
' one-based ListItems collection Index value)

Public Function ListView_GetSelectedItem(hwndLV As Long) As Long
  ListView_GetSelectedItem = ListView_GetNextItem(hwndLV, -1, LVNI_FOCUSED Or LVNI_SELECTED)
End Function

' Returns the zero-based position (index) of the item within the listview,
' that has the focus rectangle.

Public Function ListView_GetFocusedItem(hwndLV As Long) As Long
  ListView_GetFocusedItem = ListView_GetNextItem(hwndLV, -1, LVNI_FOCUSED)
End Function
 
' Selects the specified item and gives it the focus rectangle.
' does not de-select any currently selected items (user-defined).

Public Function ListView_SetSelectedItem(hwndLV As Long, i As Long) As Boolean
  g_fCodeSelection = True
  ListView_SetSelectedItem = ListView_SetItemState(hwndLV, i, LVIS_FOCUSED Or LVIS_SELECTED, _
                                                                                                     LVIS_FOCUSED Or LVIS_SELECTED)
  g_fCodeSelection = False
End Function

' Returns the zero-based position (index) of the item within the listview
' that has the specified relationship to the specified item. Emmulates
' TreeView Node (and real treeview item) relational properties (most of
' the code in this proc isn't used, but is just here to show how its done).

Public Function ListView_GetRelativeItem(hwndLV As Long, iItem As Long, dwRelative As LVRelativeItemFlags) As Long
  Dim iIndentSrc As Long
  Dim iIndentTarget As Long
  Dim i As Long
  Dim nItems As Long
  Dim iSave As Long
  
  ' get the source item's indent value, exit on failure
  iIndentSrc = -1
  Call ListView_GetItemStateEx(hwndLV, iItem, iIndentSrc)
  If (iIndentSrc = -1) Then
    ListView_GetRelativeItem = LVI_NOITEM
    Exit Function
  End If
  
  i = iItem
  nItems = ListView_GetItemCount(hwndLV)
  
  Select Case dwRelative
    
    ' =====================================================
    ' Works up the tree from the source item and returns the first item whose
    ' indent is less than the source item's indent, returns LVI_NOITEM otherwise.
    
    Case lvriParent
      Do
        i = i - 1
        If (i = LVI_NOITEM) Then Exit Do
        Call ListView_GetItemStateEx(hwndLV, i, iIndentTarget)
      Loop Until (iIndentTarget < iIndentSrc)
  
    ' =====================================================
    ' If the indent of the first item below the source item is greater than the
    ' source item's indent, returns that item, returns LVI_NOITEM otheriwse.
    
    Case lvriChild
      If (i = (nItems - 1)) Then
        i = LVI_NOITEM
      Else
        i = i + 1
        Call ListView_GetItemStateEx(hwndLV, i, iIndentTarget)
        If (iIndentTarget <= iIndentSrc) Then i = LVI_NOITEM
      End If
  
    ' =====================================================
    ' Works up the tree from the source item and returns the last item whose
    ' indent matches the source item's indent. Keeps going until an indent
    ' value less than the source item's is found. Returns the source item if it's
    ' the first sibling (not implemented in the real treeview)
    
    Case lvriFirstSibling
      iSave = i
      Do
        i = i - 1
        If (i = LVI_NOITEM) Then Exit Do
        Call ListView_GetItemStateEx(hwndLV, i, iIndentTarget)
        If (iIndentTarget = iIndentSrc) Then
          iSave = i
        ElseIf (iIndentTarget < iIndentSrc) Then
          Exit Do
        End If
      Loop
      i = iSave
      
    ' =====================================================
    ' Works down the tree from the source item and returns the last item whose
    ' indent matches the source item's indent. Keeps going until an indent
    ' value less than the source item's is found. Returns the source item if it's
    ' the last sibling (not implemented in the real treeview)
    
    Case lvriLastSibling
      iSave = i
      Do
        i = i + 1
        If (i = nItems) Then Exit Do
        Call ListView_GetItemStateEx(hwndLV, i, iIndentTarget)
        If (iIndentTarget = iIndentSrc) Then
          iSave = i
        ElseIf (iIndentTarget < iIndentSrc) Then
          Exit Do
        End If
      Loop
      i = iSave
  
    ' =====================================================
    ' Works up the tree from the source item and returns the first item whose
    ' indent matches the source item's indent. Keeps going until an indent
    ' value less than the source item's is found. Returns LVI_NOITEM if
    ' there is no previous sibling.
    
    Case lvriPrevSibling
      Do
        i = i - 1
        If (i = LVI_NOITEM) Then Exit Do
        Call ListView_GetItemStateEx(hwndLV, i, iIndentTarget)
        If (iIndentTarget = iIndentSrc) Then
          Exit Do
        ElseIf (iIndentTarget < iIndentSrc) Then
          i = LVI_NOITEM
          Exit Do
        End If
      Loop
      
    ' =====================================================
    ' Works down the tree from the source item and returns the first item whose
    ' indent matches the source item's indent. Keeps going until an indent
    ' value less than the source item's is found. Returns LVI_NOITEM if
    ' there is no next sibling.
    
    Case lvriNextSibling
      Do
        i = i + 1
        If (i = nItems) Then
          i = LVI_NOITEM
          Exit Do
        End If
        Call ListView_GetItemStateEx(hwndLV, i, iIndentTarget)
        If (iIndentTarget = iIndentSrc) Then
          Exit Do
        ElseIf (iIndentTarget < iIndentSrc) Then
          i = LVI_NOITEM
          Exit Do
        End If
      Loop
    
    ' =====================================================
    Case Else
      i = LVI_NOITEM
    
  End Select
  
  ListView_GetRelativeItem = i

End Function

' Returns the count of items within the listview that have the specified
' relationship to the specified item (this proc isn't used, but is just here
' to show how its done).

Public Function ListView_GetItemCountEx(hwndLV As Long, iItem As Long, dwRelative As LVItemCountFlags) As Long
  Dim i As Long
  Dim nItems As Long
  
  Select Case dwRelative
            
    ' =====================================================
    ' Also indicates the item's effective zero-based level in the tree hierarchy
    
    Case lvicParents
      nItems = -1
      Do
        nItems = nItems + 1
        i = ListView_GetRelativeItem(hwndLV, i, lvriParent)
      Loop Until (i = LVI_NOITEM)
      
    ' =====================================================
    Case lvrcChildren
      i = ListView_GetRelativeItem(hwndLV, i, lvriChild)
      Do Until (i = LVI_NOITEM)
        nItems = nItems + 1
        i = ListView_GetRelativeItem(hwndLV, i, lvriNextSibling)
      Loop
    
    ' =====================================================
    Case lvicSiblings
      i = ListView_GetRelativeItem(hwndLV, i, lvriFirstSibling)
      Do Until (i = LVI_NOITEM)
        nItems = nItems + 1
        i = ListView_GetRelativeItem(hwndLV, i, lvriNextSibling)
      Loop
    
  End Select
  
  ListView_GetItemCountEx = nItems

End Function

' =========================================================
' listview macros

Public Function ListView_SetImageList(hWnd As Long, hIml As Long, iImageList As Long) As Long
  ListView_SetImageList = SendMessage(hWnd, LVM_SETIMAGELIST, ByVal iImageList, ByVal hIml)
End Function
 
Public Function ListView_GetItemCount(hWnd As Long) As Long
  ListView_GetItemCount = SendMessage(hWnd, LVM_GETITEMCOUNT, 0, 0)
End Function
 
Public Function ListView_GetItem(hWnd As Long, pitem As LVITEM) As Boolean
  ListView_GetItem = SendMessage(hWnd, LVM_GETITEM, 0, pitem)
End Function
 
Public Function ListView_SetItem(hWnd As Long, pitem As LVITEM) As Boolean
  ListView_SetItem = SendMessage(hWnd, LVM_SETITEM, 0, pitem)
End Function
 
Public Function ListView_GetNextItem(hWnd As Long, i As Long, flags As Long) As Long
  ListView_GetNextItem = SendMessage(hWnd, LVM_GETNEXTITEM, ByVal i, ByVal flags)   ' ByVal MAKELPARAM(flags, 0))
End Function

Public Function ListView_GetItemRect(hWnd As Long, i As Long, prc As RECT, code As Long) As Boolean
  prc.Left = code
  ListView_GetItemRect = SendMessage(hWnd, LVM_GETITEMRECT, ByVal i, prc)
End Function

Public Function ListView_GetStringWidth(hwndLV As Long, psz As Long) As Long
  ListView_GetStringWidth = SendMessage(hwndLV, LVM_GETSTRINGWIDTH, 0, ByVal psz)
End Function

Public Function ListView_HitTest(hwndLV As Long, pinfo As LVHITTESTINFO) As Long
  ListView_HitTest = SendMessage(hwndLV, LVM_HITTEST, 0, pinfo)
End Function
 
Public Function ListView_EnsureVisible(hwndLV As Long, i As Long, fPartialOK As Boolean) As Boolean
  ListView_EnsureVisible = SendMessage(hwndLV, LVM_ENSUREVISIBLE, ByVal i, ByVal Abs(fPartialOK))   ' ByVal MAKELPARAM(Abs(fPartialOK), 0))
End Function

Public Function ListView_SetColumnWidth(hWnd As Long, iCol As Long, cx As Long) As Boolean
  ListView_SetColumnWidth = SendMessage(hWnd, LVM_SETCOLUMNWIDTH, ByVal iCol, ByVal cx)   ' ByVal MAKELPARAM(cx, 0))
End Function

Public Function ListView_SetItemState(hwndLV As Long, i As Long, state As Long, mask As Long) As Boolean
  Dim lvi As LVITEM
  lvi.state = state
  lvi.stateMask = mask
  ListView_SetItemState = SendMessage(hwndLV, LVM_SETITEMSTATE, ByVal i, lvi)
End Function

Public Function ListView_SortItems(hwndLV As Long, pfnCompare As Long, lParamSort As Long) As Boolean
  ListView_SortItems = SendMessage(hwndLV, LVM_SORTITEMS, ByVal lParamSort, ByVal pfnCompare)
End Function

' =========================================================
' imagelist macros

' Returns the one-based index of the specifed state image mask, shifted
' left twelve bits. A common control utility macro.

' Prepares the index of a state image so that a tree view control or list
' view control can use the index to retrieve the state image for an item.

Public Function INDEXTOSTATEIMAGEMASK(iIndex As Long) As Long
' #define INDEXTOSTATEIMAGEMASK(i) ((i) << 12)
  INDEXTOSTATEIMAGEMASK = iIndex * (2 ^ 12)
End Function

' Returns the state image index from the one-based index state image mask.
' The inverse of INDEXTOSTATEIMAGEMASK.

' A user-defined function (not in Commctrl.h)

Public Function STATEIMAGEMASKTOINDEX(iState As Long) As Long
  STATEIMAGEMASKTOINDEX = iState / (2 ^ 12)
End Function

' Returns the one-based index of the specified overlay image, shifted left eight bits.

Public Function INDEXTOOVERLAYMASK(iOverlay As Long) As Long
  '   INDEXTOOVERLAYMASK(i)   ((i) << 8)
  INDEXTOOVERLAYMASK = iOverlay * (2 ^ 8)
End Function
