Attribute VB_Name = "mWndProc"
Option Explicit
'
' Copyright  1997-1999 Brad Martinez, http://www.mvps.org
'
' - Code was developed using, and is formatted for, 8pt. MS Sans Serif font
'
' ==============================================
' A general purpose subclassing module w/ debugging code
' ==============================================

Public g_fCodeSelection As Boolean

Public Const WM_STYLECHANGING = &H7C
Public Const WM_KEYDOWN = &H100
Public Const WM_HSCROLL = &H114
Public Const WM_VSCROLL = &H115

' WM_STYLECHANGING/ED lParam
Public Type STYLESTRUCT   ' ss
  styleOld  As Long
  styleNew  As Long
End Type

' WM_H/VSCROLL wParam
Public Const SB_LINEUP = 0
Public Const SB_LINELEFT = 0
Public Const SB_LINEDOWN = 1
Public Const SB_LINERIGHT = 1

' OCM_NOTIFY is WM_NOTIFY reflected to a C++ created ActiveX control.
' http://msdn.microsoft.com/library/devprods/vs6/visualc/vccore/_core_activex_controls.3a_.subclassing_a_windows_control.htm
Public Const WM_NOTIFY = &H4E
Public Const WM_USER = &H400
Public Const OCM__BASE = (WM_USER + &H1C00)
Public Const OCM_NOTIFY = (OCM__BASE + WM_NOTIFY)

' The NMHDR structure contains information about a notification message. The pointer
' to this structure is specified as the lParam member of the WM_NOTIFY message.
Public Type NMHDR
  hwndFrom As Long   ' Window handle of control sending message
  idFrom As Long        ' Identifier of control sending message
  code  As Long          ' Specifies the notification code
End Type

Private Const WM_DESTROY = &H2

Declare Sub MoveMemory Lib "kernel32" Alias "RtlMoveMemory" (pDest As Any, pSource As Any, ByVal dwLength As Long)
Declare Function GetProp Lib "user32" Alias "GetPropA" (ByVal hWnd As Long, ByVal lpString As String) As Long
Declare Function SetProp Lib "user32" Alias "SetPropA" (ByVal hWnd As Long, ByVal lpString As String, ByVal hData As Long) As Long
Declare Function RemoveProp Lib "user32" Alias "RemovePropA" (ByVal hWnd As Long, ByVal lpString As String) As Long

Public Enum GWL_nIndex
  GWL_WNDPROC = (-4)
'  GWL_HWNDPARENT = (-8)
  GWL_ID = (-12)
  gwl_style = (-16)
  GWL_EXSTYLE = (-20)
'  GWL_USERDATA = (-21)
End Enum

Declare Function GetWindowLong Lib "user32" Alias "GetWindowLongA" (ByVal hWnd As Long, ByVal nIndex As GWL_nIndex) As Long
Declare Function SetWindowLong Lib "user32" Alias "SetWindowLongA" (ByVal hWnd As Long, ByVal nIndex As GWL_nIndex, ByVal dwNewLong As Long) As Long

Declare Function CallWindowProc Lib "user32" Alias "CallWindowProcA" (ByVal lpPrevWndFunc As Long, ByVal hWnd As Long, ByVal uMsg As Long, ByVal wParam As Long, ByVal lParam As Long) As Long
Declare Function DefWindowProc Lib "user32" Alias "DefWindowProcA" (ByVal hWnd As Long, ByVal wMsg As Long, ByVal wParam As Long, ByVal lParam As Long) As Long

Private Const OLDWNDPROC = "OldWndProc"
Private Const OBJECTPTR = "ObjectPtr"

#If DEBUGWINDOWPROC Then
  ' maintains a WindowProcHook reference for each subclassed window.
  ' the window's handle is the collection item's key string.
  Private m_colWPHooks As New Collection
#End If
'

Public Function SubClass(hWnd As Long, _
                                         lpfnNew As Long, _
                                         Optional objNotify As Object = Nothing) As Boolean
  Dim lpfnOld As Long
  Dim fSuccess As Boolean
  On Error GoTo Out

  If GetProp(hWnd, OLDWNDPROC) Then
    SubClass = True
    Exit Function
  End If
  
#If (DEBUGWINDOWPROC = 0) Then
    lpfnOld = SetWindowLong(hWnd, GWL_WNDPROC, lpfnNew)

#Else
    Dim objWPHook As WindowProcHook
    
    Set objWPHook = CreateWindowProcHook
    m_colWPHooks.Add objWPHook, CStr(hWnd)
    
    With objWPHook
      Call .SetMainProc(lpfnNew)
      lpfnOld = SetWindowLong(hWnd, GWL_WNDPROC, .ProcAddress)
      Call .SetDebugProc(lpfnOld)
    End With

#End If
  
  If lpfnOld Then
    fSuccess = SetProp(hWnd, OLDWNDPROC, lpfnOld)
    If (objNotify Is Nothing) = False Then
      fSuccess = fSuccess And SetProp(hWnd, OBJECTPTR, ObjPtr(objNotify))
    End If
  End If
  
Out:
  If fSuccess Then
    SubClass = True
  
  Else
    If lpfnOld Then Call SetWindowLong(hWnd, GWL_WNDPROC, lpfnOld)
    MsgBox "Error subclassing window &H" & Hex(hWnd) & vbCrLf & vbCrLf & _
                  "Err# " & Err.Number & ": " & Err.Description, vbExclamation
  End If
  
End Function

Public Function UnSubClass(hWnd As Long) As Boolean
  Dim lpfnOld As Long
  
  lpfnOld = GetProp(hWnd, OLDWNDPROC)
  If lpfnOld Then
    
    If SetWindowLong(hWnd, GWL_WNDPROC, lpfnOld) Then
      Call RemoveProp(hWnd, OLDWNDPROC)
      Call RemoveProp(hWnd, OBJECTPTR)

#If DEBUGWINDOWPROC Then
      ' remove the WindowProcHook reference from the collection
      m_colWPHooks.Remove CStr(hWnd)
#End If
      
      UnSubClass = True
    
    End If   ' SetWindowLong
  End If   ' lpfnOld

End Function

' Processes listview window messages.

Public Function LVWndProc(ByVal hWnd As Long, ByVal uMsg As Long, ByVal wParam As Long, ByVal lParam As Long) As Long
  
  Select Case uMsg

    ' ============================================================
    ' For some reason the ListView toggles it's LVS_NOLABELWRAP style bit
    ' before the ListItems.Add method is called, and toggles it back after one or
    ' more consecutive ListItems.Adds. But changing the style causes the ListView
    ' to scroll the focused item into view, which we obviously don't want. So, when
    ' we see this bit being toggled, prevent it, and be done with it...
    
    Case WM_STYLECHANGING
      Dim ss As STYLESTRUCT
      
      If (wParam = gwl_style) Then
        MoveMemory ss, ByVal lParam, Len(ss)
        If (Abs(ss.styleOld - ss.styleNew) = LVS_NOLABELWRAP) Then
          ss.styleNew = ss.styleOld
          MoveMemory ByVal lParam, ss, Len(ss)
        End If
      End If
      
    ' ============================================================
    ' Handle ListView keyboard input
    
    Case WM_KEYDOWN
      Dim fDidScroll As Boolean
      
      ' If the Control key is pressed, provide treeview-like Ctrl+Arrow scrolling
      ' without changing the selected ListView item
      If (wParam <> vbKeyControl) And (GetKeyState(vbKeyControl) And &H8000) Then
        Select Case wParam
          Case vbKeyLeft
            Call SendMessage(hWnd, WM_HSCROLL, SB_LINELEFT, ByVal 0&)
            fDidScroll = True
          Case vbKeyRight
            Call SendMessage(hWnd, WM_HSCROLL, SB_LINERIGHT, ByVal 0&)
            fDidScroll = True
          Case vbKeyUp
            Call SendMessage(hWnd, WM_VSCROLL, SB_LINEUP, ByVal 0&)
            fDidScroll = True
          Case vbKeyDown
            Call SendMessage(hWnd, WM_VSCROLL, SB_LINEDOWN, ByVal 0&)
            fDidScroll = True
        End Select
        
        ' Prevents the arrow key selection change
        If fDidScroll Then Exit Function
        
      ' By default the listview scrolls its client window on vbKeyRight and vbKeyLeft,
      ' but we don't want that. So process the keystroke in ListView1_KeyDown and
      ' eat the messeage (don't call CallWindowProc), preventing the scroll.
      ElseIf (wParam = vbKeyRight) Or (wParam = vbKeyLeft) Then
        Call Form1.ListView1_KeyDown((wParam), 0)
        Exit Function
      
      End If   ' (GetKeyState(vbKeyControl) And &H8000)
      
    ' ============================================================
    ' Process ListView notification messages reflected back to the ListView from
    ' it's parent OLE control reflector window.

    Case OCM_NOTIFY
      Dim nmh As NMHDR
      Dim nmlv As NMLISTVIEW
    
      MoveMemory nmh, ByVal lParam, Len(nmh)
      Select Case nmh.code
        
        ' For normal listviews, and the Comctl32.ocx ListView (but not the Mscomvtl.ocx
        ' ListView), when the mouse, keyboard or code casues a selection change,
        ' 3 LVN_ITEMCHANGING/LVIF_STATE (nmlv.uChanged) notifications are
        ' sent in the following sequence:
        '   old item :   nmlv.uOldState = LVIS_FOCUSED,   nmlv.uNewState = 0
        '   old item :   nmlv.uOldState = LVIS_SELECTED, nmlv.uNewState = 0
        '   new item : nmlv.uOldState = 0,                            nmlv.uNewState = LVIS_FOCUSED Or LVIS_SELECTED
        '
        ' So, if the old item's LVIS_SELECTED is being removed and it still has
        ' LVIS_FOCUSED, then know that the focused item is being de-selected,
        ' which we don't want (unless we're doing it in code) since we're replicating
        ' treeview functionality (always has a selected item). So we'll cancel the
        ' notification unless we set the flag.
        Case LVN_ITEMCHANGING
          
          If (g_fCodeSelection = False) Then
            MoveMemory nmlv, ByVal lParam, Len(nmlv)
            
            If (nmlv.uOldState = LVIS_SELECTED) And (nmlv.uNewState = 0) Then
              If (ListView_GetFocusedItem(hWnd) = nmlv.iItem) Then
                LVWndProc = 1
                Exit Function
              End If
            End If
          End If
          
        ' Remove the open icon from the previously focused item, and add the open
        ' icon to the new focused item.
        Case LVN_ITEMCHANGED
          Dim lvi As LVITEM
          
          MoveMemory nmlv, ByVal lParam, Len(nmlv)
          
          If (nmlv.uOldState And LVIS_FOCUSED) Or (nmlv.uNewState And LVIS_FOCUSED) Then
            lvi.mask = LVIF_IMAGE
            lvi.iItem = nmlv.iItem
            ' Since we didn't sort the ListView using the API, all ListItem Index
            ' values correspond to each ListItem's one-based position within
            ' the ListView (a value of 1 greater than the real listview item index)
            lvi.iImage = GetFileIconIndex(Form1.ListView1.ListItems(nmlv.iItem + 1).Tag, _
                                                          SHGFI_SMALLICON Or (SHGFI_OPENICON And CBool(nmlv.uNewState And LVIS_FOCUSED)))
            Call ListView_SetItem(hWnd, lvi)
          End If
          
        End Select
        
    ' ============================================================
    ' Prevent the ListView from removing our system imagelist assignment, which
    ' it wil do when it sees no VB ImageList associated with it.
    ' (the ListView can't be subclassed when we're assigning imagelists...)
    
    Case LVM_SETIMAGELIST
      Exit Function
          
    ' ============================================================
    ' Unsubclass the window if we forget to do it...
    
    Case WM_DESTROY
      ' OLDWNDPROC will be gone after UnSubClass is called!
      Call CallWindowProc(GetProp(hWnd, OLDWNDPROC), hWnd, uMsg, wParam, lParam)
      Call UnSubClass(hWnd)
      Exit Function
  
  End Select
  
  LVWndProc = CallWindowProc(GetProp(hWnd, OLDWNDPROC), hWnd, uMsg, wParam, lParam)

End Function
